@@ -4,6 +4,6 @@ if (requireNamespace("Infusion",quietly=TRUE)) {
   Infusion::Infusion.options(spaMM.options("example_maxtime"))
   #Infusion.options(example_maxtime=20)  ## allows basic example
   #Infusion.options(example_maxtime=120)  ## allows refine()
-  example("Infusion",package="Infusion",ask=FALSE) ## detected calc_logdisp_cov issues   
+  example("example_raw",package="Infusion",ask=FALSE) ## detected calc_logdisp_cov issues   
 }
 
