@@ -26,7 +26,8 @@ if( requireNamespace("multilevel", quietly = TRUE)) {
     lql$resp <- as.integer(lql$dichot)-1
     # 12s s WITHOUT LevenbergM NOR sparse_precision
     hl1 <- HLfit(cbind(resp,1-resp) ~ 0+itype+(1|subj)+(1|COMPID)+(1|item), 
-                 data=lql, family=binomial() ,HLmethod="ML",verbose=c(TRACE=interactive()),control.HLfit=list(LevenbergM=FALSE)) 
+                 data=lql, family=binomial() ,HLmethod="ML",verbose=c(TRACE=interactive()),
+                 control.HLfit=list(LevenbergM=FALSE)) 
     testthat::expect_equal(logLik(hl1),c(p_v=-20354.8828))
     if (FALSE) { ## not useful in routine tests
       if (file.exists((privtest <- "C:/home/francois/travail/stats/spaMMplus/spaMM/package/tests_other_pack/test-Rasch-lme4.R"))) {
